#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
from collections import namedtuple
from typing import Dict, List, Optional, Sequence, Tuple

import numpy as np


EditDistanceResult = namedtuple('EditDistanceResult',
                                ('distance', 'alignment'))

OPERATIONS = {'n': 0, 's': 1, 'd': 1, 'i': 1}


def edit_distance(gold: Sequence, hyp: Sequence, operations:
                  Optional[Dict[str, int]] = None) -> EditDistanceResult:
    """Calculate edit distance and one alignment.

    The algorithm used is Wagner-Fischer.

    :param gold: The source sequence.
    :param hyp: The target sequence.
    :param operations: Dict with costs for the operations
        'n', 's', 'i' and 'd'.
        The algorithm only works if operations['n'] < operations['s'].
    :return: Edit distance and alignment as a namedtuple
    """
    # Avoid dict as default parameter because it is mutable.
    # Use None instead and set it explicitly.
    if operations is None:
        operations = OPERATIONS

    costs = np.zeros((len(gold) + 1, len(hyp) + 1), dtype=int)

    # Deletions necessary to get from gold to empty hyp.
    for i in range(len(gold) + 1):
        costs[i, 0] = i * operations['d']

    # Insertions necessary to get from empty gold to hyp.
    for j in range(len(hyp) + 1):
        costs[0, j] = j * operations['i']

    # Fill matrix starting in the top left corner.
    for j in range(1, len(hyp) + 1):
        for i in range(1, len(gold) + 1):
            if gold[i - 1] == hyp[j - 1]:
                costs[i, j] = costs[i - 1, j - 1] + operations['n']
            else:
                costs[i, j] = min(
                    costs[i - 1, j - 1] + operations['s'],
                    costs[i, j - 1] + operations['i'],
                    costs[i - 1, j] + operations['d'],
                )

    # Trace back the path through the matrix to find an alignment.
    alignment = []
    i = len(gold)
    j = len(hyp)
    while i != 0 or j != 0:
        di, dj = min([(1, 1), (0, 1), (1, 0)],
                     key=(lambda delta: costs[i - delta[0], j - delta[1]]))
        if di == 1 and dj == 1:
            if costs[i - di, j - dj] + OPERATIONS['s'] == costs[i, j]:
                # It was a substitution.
                alignment.append('s')
            else:
                # It was a null operation.
                alignment.append('n')
        elif di == 1 and dj == 0:
            # It was a deletion.
            alignment.append('d')
        else:
            # It was an insertion.
            alignment.append('i')
        i -= di
        j -= dj

    alignment = ''.join(reversed(alignment))
    result = EditDistanceResult(distance=costs[-1, -1], alignment=alignment)
    return result


def tokenize(string: str, separator: str = ' ') -> List[str]:
    """Tokenize a string by splitting on a separator.

    :param string: The string that is to be split.
    :param separator: The separator to use for splitting.
    :return: A list of the tokens.
    """
    return [token for token in string.split(separator) if token]


def get_word_error_rate(gold_token_count: int, error_count: int) -> float:
    """Calculate the word error rate.

    :param gold_token_count: Number of tokens in the gold sequence.
    :param error_count: Number of errors made.
    :return: The word error rate.
    """
    if gold_token_count == 0:
        return None
    else:
        return error_count / gold_token_count


def main(gold: str, hypothesis: str,
         print_: bool = True) -> Tuple[EditDistanceResult, float]:
    """Calculate edit distance and word error rate for two sentences.

    :param gold: The reference sentence.
    :param hypothesis: The hypothesis.
    :return: edit distance result and word error rate as a tuple.
    """
    hyp_tokens = tokenize(hypothesis)
    gold_tokens = tokenize(gold)
    result = edit_distance(hyp=hyp_tokens, gold=gold_tokens)
    word_error_rate = get_word_error_rate(len(gold_tokens), result.distance)
    if print_:
        print('Edit Distance: {r.distance}\n'
              'Alignment: {r.alignment}\n'
              'Word Error Rate: {wer:.2f}'
              .format(r=result, wer=word_error_rate))
    return result, word_error_rate


def parse_args():
    description = ('Calculate the edit distance between the tokens in two'
                   ' strings.')
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('gold')
    parser.add_argument('hypothesis')
    args = parser.parse_args()
    return args


if __name__ == '__main__':
    ARGS = parse_args()
    main(**vars(ARGS))
