#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest

from edit_distance import (edit_distance, get_word_error_rate, main,
                           OPERATIONS, tokenize)


class TestEditDistance(unittest.TestCase):

    def test_matching(self):
        result = edit_distance(hyp=['a', 'a', 'a'], gold=['a', 'a', 'a'],
                               operations=OPERATIONS)
        self.assertEqual(result.distance, 0)

    def test_empty(self):
        result = edit_distance(hyp=[], gold=[])
        self.assertEqual(result.distance, 0)

    def test_s_and_d(self):
        result = edit_distance(hyp=['a', 'b'], gold=['a', 'a', 'a'],
                               operations=OPERATIONS)
        expected_distance = OPERATIONS['s'] + OPERATIONS['d']
        self.assertEqual(result.distance, expected_distance)

    def test_s_and_i(self):
        result = edit_distance(hyp=['a', 'b', 'c', 'a'], gold=['a', 'a', 'a'],
                               operations=OPERATIONS)
        expected_distance = OPERATIONS['s'] + OPERATIONS['i']
        self.assertEqual(result.distance, expected_distance)


class TestTokenize(unittest.TestCase):

    def test_empty(self):
        self.assertEqual(tokenize(''), [])

    def test_simple_cases(self):
        self.assertEqual(tokenize('a a a'), ['a', 'a', 'a'])
        self.assertEqual(tokenize('a b'), ['a', 'b'])
        self.assertEqual(tokenize('a b c a'), ['a', 'b', 'c', 'a'])

    def test_avoid_empty_tokens(self):
        self.assertEqual(tokenize('a  b'), ['a', 'b'])
        self.assertEqual(tokenize('   '), [])


class TestGetWordErrorRate(unittest.TestCase):

    def test_empty_gold(self):
        self.assertIsNone(get_word_error_rate(0, 'should be irrelevant'))

    def test_normal_case(self):
        self.assertEqual(get_word_error_rate(10, 4), 0.4)

    def test_bad_case(self):
        self.assertEqual(get_word_error_rate(10, 12), 1.2)


class IntegrationTest(unittest.TestCase):

    def test_matching(self):
        hypothesis = 'a a a'
        gold = 'a a a'
        result, word_error_rate = main(gold=gold, hypothesis=hypothesis,
                                       print_=False)
        self.assertEqual(result.distance, 0)
        self.assertEqual(word_error_rate, 0.0)

    def test_empty(self):
        hypothesis = ''
        gold = ''
        result, word_error_rate = main(gold=gold, hypothesis=hypothesis,
                                       print_=False)
        self.assertEqual(result.distance, 0)
        self.assertIsNone(word_error_rate)

    def test_s_and_d(self):
        hypothesis = 'a b'
        gold = 'a a a'
        expected_distance = OPERATIONS['s'] + OPERATIONS['d']
        result, word_error_rate = main(gold=gold, hypothesis=hypothesis,
                                       print_=False)
        self.assertEqual(result.distance, expected_distance)
        self.assertEqual(word_error_rate, 2/3)

    def test_s_and_i(self):
        hypothesis = 'a b c a'
        gold = 'a a a'
        expected_distance = OPERATIONS['s'] + OPERATIONS['i']
        result, word_error_rate = main(gold=gold, hypothesis=hypothesis,
                                       print_=False)
        self.assertEqual(result.distance, expected_distance)
        self.assertEqual(word_error_rate, 2/3)


if __name__ == '__main__':
    unittest.main()
