#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
from typing import Sequence

import kenlm


def cross_entropy(lm: kenlm.Model, token_count: int,
                  sentences: Sequence[str]) -> float:
    """Calculate the cross entropy of a language model on a text.
    
    :param lm: The language model to use.
    :param token_count: Number of tokens in the text.
    :param sentences: The sentences making up the text.
    :return: The calculated cross entropy."""
    xent = - sum(
        score
        for sentence in sentences
        for score, _, _ in lm.full_scores(sentence)
    ) / token_count
    return xent


def unigram_cross_entropy(lm: kenlm.Model, token_count: int,
                          sentences: Sequence[str]) -> float:
    """Calculate the cross entropy of a language model on a text.
    
    :param lm: The language model to use.
    :param token_count: Number of tokens in the text.
    :param sentences: The sentences making up the text.
    :return: The calculated cross entropy."""
    xent = - sum(
        lm.score(token)
        for sentence in sentences
        for token in sentence.split()
    ) / token_count
    return xent


def main(model_file, text_file):
    lm = kenlm.LanguageModel(model_file)

    sentences = []
    token_count = 0
    with open(text_file) as f:
        for line in f:
            sentence = line.strip()
            sentences.append(sentence)
            token_count += len(sentence.split())

    xent = cross_entropy(lm, token_count, sentences)
    uxent = unigram_cross_entropy(lm, token_count, sentences)
    print('Sentence Count: {}\nToken Count: {}\nCross Entropy: {}\n'
          'Unigram Cross Entropy: {}'
          .format(len(sentences), token_count, xent, uxent))


def parse_args():
    description = ('Calculate the cross entropy of a Ken Language Model'
                   ' on a text file.')
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('model_file', help='File holding a KenLM')
    parser.add_argument('text_file',
        help='A text file to calculate the cross entropy on')
    args = parser.parse_args()
    return args


if __name__ == '__main__':
    ARGS = parse_args()
    main(**vars(ARGS))
